@@ -6,6 +6,7 @@ import os
 import datetime
 import platform
 import csv
+import base64
 
 
 class Query(object):
@@ -15,7 +16,7 @@ class Query(object):
     HEADERS = {
         "Accept": "application/json",
         "Content-Type": "application/json",
-        "User-Agent": "Python API 0.33 / {local_version}".format(local_version=platform.python_version())
+        "User-Agent": "Python API 0.36 / {local_version}".format(local_version=platform.python_version())
     }
 
     def __init__(self, proxies={}, verify=True):
@@ -451,7 +452,7 @@ class Query(object):
         return self._parse_response(response.text, 'deleteTable')
 
     # CreateColumn
-    def create_column(self, project_id, table_id, display_name, data_type, linked_table=None, is_required=False):
+    def create_column(self, project_id, table_id, display_name, data_type, linked_context=None, linked_table=None, is_required=False):
         if not project_id:
             raise Exception('Project ID required')
         if not table_id:
@@ -460,7 +461,7 @@ class Query(object):
             raise Exception('Display Name required')
         if not data_type:
             raise Exception('Data Type required')
-        if data_type not in ['text', 'number', 'boolean', 'url', 'date', 'date_added', 'link_one', 'link_many', 'image', 'image_url']:
+        if data_type not in ['text', 'number', 'boolean', 'url', 'date', 'date_added', 'date_updated', 'auto_id', 'link_one', 'link_many', 'image', 'image_url']:
             raise Exception('Invalid Data Type')
         operation_name = 'CreateColumn'
         response = self._requests(
@@ -473,8 +474,9 @@ class Query(object):
                     "data": {
                         "displayName": display_name,
                         "dataType": data_type,
-                        "linkedTable": linked_table,
-                        "isRequired": is_required
+                        "isRequired": is_required,
+                        "linkedContext": linked_context,
+                        "linkedTable": linked_table
                     } if data_type in ['link_one', 'link_many'] else {
                         "displayName": display_name,
                         "dataType": data_type,
@@ -488,7 +490,7 @@ class Query(object):
         return self._parse_response(response.text, 'createColumn')
 
     # UpdateColumn
-    def update_column(self, project_id, table_id, column_id, display_name, data_type, linked_table=None, restore_relationships_data=False):
+    def update_column(self, project_id, table_id, column_id, display_name, data_type, linked_context=None, linked_table=None, is_required=False, restore_relationships_data=True):
         if not project_id:
             raise Exception('Project ID required')
         if not table_id:
@@ -499,7 +501,7 @@ class Query(object):
             raise Exception('Display Name required')
         if not data_type:
             raise Exception('Data Type required')
-        if data_type not in ['text', 'number', 'boolean', 'url', 'date', 'date_added', 'link_one', 'link_many', 'image', 'image_url']:
+        if data_type not in ['text', 'number', 'boolean', 'url', 'date', 'date_added', 'date_updated', 'auto_id', 'link_one', 'link_many', 'image', 'image_url']:
             raise Exception('Invalid Data Type')
         operation_name = 'UpdateColumn'
         response = self._requests(
@@ -512,9 +514,16 @@ class Query(object):
                     "columnId": column_id,
                     "data": {
                         "displayName": display_name,
-                        "dataType": data_type
+                        "dataType": data_type,
+                        "isRequired": is_required,
+                        "linkedContext": linked_context,
+                        "linkedTable": linked_table
+                    } if data_type in ['link_one', 'link_many'] else {
+                        "displayName": display_name,
+                        "dataType": data_type,
+                        "isRequired": is_required
                     },
-                    "restoreRelationshipsData": False
+                    "restoreRelationshipsData": restore_relationships_data
                 },
                 "operationName": operation_name
             }
@@ -546,6 +555,32 @@ class Query(object):
         self._validate_response(response.text, 'deleteColumn')
         return self._parse_response(response.text, 'deleteColumn')
 
+    # UploadFile
+    def upload_file(self, filepath):
+        if not os.path.exists(filepath):
+            raise Exception('File not found')
+        ext = os.path.splitext(filepath)[-1].lower().replace('.', '')
+        img_file = open(filepath, 'rb')
+        encoded_string = base64.b64encode(img_file.read())
+        operation_name = 'UploadFile'
+        response = self._requests(
+            method='post',
+            json={
+                "query": self._get_query(type='mutation', name='UploadFile'),
+                "variables": {
+                    "namespace":"files",
+                    "data":"data:image/{};base64,{}".format(
+                        ext,
+                        encoded_string
+                    )
+                },
+                "operationName": operation_name
+            }
+        )
+        self._validate_response(response.text, 'uploadFile')
+        result = self._parse_response(response.text, 'uploadFile')
+        return result['path']
+
     # CreateVertex
     def create_vertex(self, project_id, table_id, values, edges):
         if not project_id:
@@ -569,10 +604,11 @@ class Query(object):
             })
 
         data_edges = []
-        for label, to_id in edges:
+        for edge in edges:
             data_edges.append({
-                "label": label,
-                "toId": to_id
+                "label": edge["label"],
+                "toContext": edge["to_context"],
+                "toId": edge["to_id"]
             })
 
         operation_name = 'CreateVertex'
@@ -619,10 +655,11 @@ class Query(object):
             })
 
         data_edges = []
-        for label, to_id in edges:
+        for edge in edges:
             data_edges.append({
-                "label": label,
-                "toId": to_id
+                "label": edge["label"],
+                "toContext": edge["to_context"],
+                "toId": edge["to_id"]
             })
 
         operation_name = 'UpdateVertex'
